/*******************************************************************************
 * WRW - WorldRadio Widget
 * by Alexandre Bazyl <wrw@bazyl.com.br>
 *
 * -- version 1.3   :: JUN/2008
 * -- version 1.1.4 :: OCT/2007
 * -- version 1.1.3 :: OCT/2007
 * -- version 1.1.2 :: SEP/2007
 * -- version 1.0b  :: AUG/2007
 *
 * wrcontroller.js - funcoes relacionadas ao objeto Controller
 *
 *******************************************************************************/

// Variaveis globais
var myXMLRequest;

/********************************* CONTROLLER **********************************/
Controller = new Object();
// Controle de Popup
Controller.clearPopup = controller_clearPopup;
Controller.setPopupText = controller_setPopupText;
// Controle de Paises
Controller.countryAdd = controller_addCountry;
Controller.countryUpdate = controller_updateCountry;
Controller.countryDel = controller_deleteCountry;
Controller.countryChangeBackPopup = controller_changeBackCountry;
Controller.countryChangeFrontPopup = controller_changeFrontCountry;
// Controle de Radios
Controller.radioAdd = controller_addRadio;
Controller.radioUpdate = controller_updateEntryRadio;
Controller.radioDel = controller_deleteRadio;
Controller.radioChangeBackPopup = controller_changeBackRadio;
Controller.radioChangeFrontPopup = controller_changeFrontRadio;
// Controle de persistencia
Controller.save = controller_save;
Controller.subSave = controller_subSave;
Controller.load = controller_loadAll;
// Controle de Atualizacoes
Controller.checkWidgetUpdate = controller_checkWidgetUpdate;
Controller.checkRadioUpdate = controller_checkRadioUpdate;
Controller.cancelUpdate = controller_cancelUpdate;
Controller.acceptUpdate = controller_acceptUpdate;
Controller.checkUpdates = controller_checkUpdates;
Controller.receiveXMLRequest = controller_receiveXMLRequest;
Controller.processXMLRequest = controller_processXMLRequest;
Controller.processRadiosXMLRequest = controller_processRadiosXMLRequest;
Controller.processUpdateXMLRequest = controller_processUpdateXMLRequest;
Controller.updateWidget = controller_updateWidget;
Controller.updateRadios = controller_updateRadios;
Controller.openHomePageWRW = controller_openHomePage;
// Atributos
Controller.downloadStep = 0;	//dlstep;
Controller.downloadFunction = 0;

var VRF_WDGT  = 1;
var VRF_RADIO = 2;
var DWN_RADIO = 3;

/************************************ PARSER ************************************/
Parser = new Object();
Parser.findChild = parser_findChild;
Parser.checkString = parser_checkString;
Parser.decodeURL = parser_decodeURL;
Parser.processURL = parser_processURL;
Parser.correctString = parser_correctString;
Parser.validateMetadata = parser_validateMetadata;

// Atributos
//Parser.uri = "";


/***************************** CONTROLLER-PAISES *******************************/

/*******************************************************************************
 * controller_addCountry - funcao que controla a insercao de um novo pais
 *******************************************************************************/ 
function controller_addCountry() 
{
	var nome = document.getElementById("txt_countryName").value;
	var menu = document.getElementById("popup_backCountry");
	 
	// VRF se a caixa de texto esta preenchida 
	if (!Parser.checkString(nome)) {
		Panel.backShowMessage("Invalid country name!");
		return;
	}
	// VRF se existe um pais com o mesmo nome
	if (Country.search(nome)!=-1) {
		Panel.backShowMessage("Country already exists!");
		return;
	}
	
	// Insere o novo pais na estrutura paises
	var num_pais = Country.add(nome, -1);		// -1 para indicar que eh uma nova numeracao
	
	// Limpa e preenche novamente o popup dos paises
	Country.populate("popup_backCountry", num_pais);
	
	// Atualiza caixa de texto txt_countryName
	document.getElementById("txt_countryName").value = "";
	
	Config.isChangedCountry = true;
}


/*******************************************************************************
 * controller_updateCountry - funcao que controla a atualizacao de um pais
 *******************************************************************************/
function controller_updateCountry() 
{
	var box = document.getElementById("popup_backCountry");
	if (box.length == 0) return;
	
	// VRF se a caixa de texto estah preenchida
	var novo_nomepais = document.getElementById("txt_countryName").value;
	if (!Parser.checkString(novo_nomepais)) {
		Panel.backShowMessage("Invalid country name!");
		return;
	}
	// Verifica se jah existe um pais com o mesmo nome
	if (Country.search(novo_nomepais)!=-1) {
		Panel.backShowMessage("Country already exists!");
		return;
	}

	var num_pais = box.options[box.selectedIndex].value;		// Numero do pais
	// Atualiza a struct paises (remove e insere com a mesma numeracao)
	Country.del(num_pais);
	Country.add(novo_nomepais, num_pais);

	// Limpa e preenche novamente o popup dos paises
	Country.populate("popup_backCountry", num_pais);
	
	Config.isChangedCountry = true;
}


/*******************************************************************************
 * controller_deleteCountry - funcao que controla a remocao de um pais
 *******************************************************************************/
function controller_deleteCountry() 
{
	var box = document.getElementById("popup_backCountry");
	if (box.length==0) return;

	var num_pais = box.options[box.selectedIndex].value;
	
	// Temporario - se o pais possuir radios, entao retorna
	for (var i=1; i<=radios[0].num; i++)
		if(radios[i].pais == num_pais) {
			Panel.backShowMessage("The country can't be deleted!");
			return;
		}
	// End temporario
	
	// Remove da struct paises
	Country.del(num_pais);
	document.getElementById("txt_countryName").value = "";			
	// Limpa e preenche novamente o popup dos paises
	Country.populate("popup_backCountry", -1);
	
	Config.isChangedCountry = true;
}


/*******************************************************************************
 * controller_changeBackCountry - Evento onChange na popup_backCountry (Back)
 *                      Atualiza a caixa de texto com o nome do pais e a popup
 *                      das radios deste pais
 *******************************************************************************/
function controller_changeBackCountry()
{
	var box = document.getElementById("popup_backCountry");
	if (box.length==0) {
		document.getElementById("lbl_countryname").innerHTML = "None";
		return;
	}
	var chosen = box.selectedIndex;
	var numpais = box.options[chosen].value;
	
	// Atualiza caixa de texto referente a pais
	document.getElementById("txt_countryName").value = box.options[chosen].text;

	// Atualiza informacoes das Radios deste pais
	// Atualiza o popup de radios
	Radio.populate("popup_backRadio", numpais, -1);
	// Atualiza as caixas de texto referentes a radio
	document.getElementById("lbl_countryname").innerHTML = box.options[chosen].text;
	radio_popup = document.getElementById("popup_backRadio");
	if (radio_popup.length > 0) {
		var numradio = radio_popup.options[radio_popup.selectedIndex].value;
		document.getElementById("txt_radioName").value = Radio.getAttrib(numradio, "nome");
		document.getElementById("txt_radioURL").value = Radio.getAttrib(numradio, "link");
	}
	else {
		document.getElementById("txt_radioName").value = "";
		document.getElementById("txt_radioURL").value = "";
	}
}


/*******************************************************************************
 * controller_changeFrontCountry - Evento onChange na popup_plCountry (Front)
 *                    Atualiza a popup das radios do pais selecionado e os
 *                    botoes de memoria
 *******************************************************************************/
function controller_changeFrontCountry()
{
	var box = document.getElementById("popup_plCountry");
	var numpais = box.options[box.selectedIndex].value;
	
	// Atualiza Radios
	Radio.populate("popup_plRadio", numpais, -1);	// Atualiza o popup com as radios deste pais
	Memory.refreshButtons();						// Atualiza os botoes de memorias
	Memory.onForCurrentRadio();			// VRF se a radio possui um botao de memo atribuido a ela
	Controller.setPopupText();
	
	box_radio = document.getElementById("popup_plRadio");
	if (box_radio.length>0)
		Memory.radioSelected = box_radio.options[box_radio.options.selectedIndex].value;
	else
		Memory.radioSelected=0;
}


/***************************** CONTROLLER-RADIOS *******************************/

/*******************************************************************************
 * controller_addRadio - funcao que controla a insercao de uma nova radio
 *******************************************************************************/
function controller_addRadio() 
{
    if (paises[0].num==0) {
		Panel.BackShowMessage("None country exists!");
		return;
	}
	var nome = document.getElementById("txt_radioName").value;
	var r_url = document.getElementById("txt_radioURL").value;
	// VRF se a caixa de texto esta preenchida e
	// se nao existe uma radio com o mesmo nome
	if (!Parser.checkString(nome) || !Parser.checkString(r_url)) {
		Panel.BackShowMessage("Invalid URL or radio name!");
		return;
	}
	if (Radio.search(nome)!=-1) {
		Panel.backShowMessage("Radio already exists!");
		return;
	}
		
	var pais_root = document.getElementById("popup_backCountry");
	var num_pais = pais_root[pais_root.selectedIndex].value;
	
	// Atualiza a struct radios
	Radio.add(nome, r_url, num_pais, -1);
	
	// Atualiza as caixas de texto
	document.getElementById("txt_radioName").value = "";
	document.getElementById("txt_radioURL").value = "";
	
	// Limpa e preenche novamente o popup das radios
	Radio.populate("popup_backRadio", num_pais, -1);
	
	Config.isChangedRadio = true;
}


/*******************************************************************************
 * controller_updateEntryRadio - funcao que controla a atualizacao de uma radio
 *******************************************************************************/
function controller_updateEntryRadio() 
{
	var box = document.getElementById("popup_backRadio");
	if (box.length == 0) return;
	
	var nnome = document.getElementById("txt_radioName").value;
	var r_url = document.getElementById("txt_radioURL").value;
	var num = box.options[box.selectedIndex].value;
	
	// VRF se a caixa de texto esta preenchida
	if (!Parser.checkString(nnome) || !Parser.checkString(r_url)) {
		Panel.backShowMessage("Invalid URL or radio name!");
		return;
	}
	
	// Aqui tem um bug: se o usuario mudar a radio no popup e digitar um nome
	// que jah existe, o widget vai aceitar...
	// VRF se nao existe uma radio com o mesmo nome
	var chk = Country.search(nnome);
	if (chk!=-1 && chk!=num) {
		Panel.backShowMessage("Radio already exists!");
		return;
	}
	
	// Numero do pais
	var pais_root = document.getElementById("popup_backCountry");
	var num_pais = pais_root[pais_root.selectedIndex].value;
	
	// Atualiza a struct radios
	Radio.del(num);
	Radio.add(nnome, r_url, num_pais, num);
		
	// Limpa e preenche novamente o popup das radios
	Radio.populate("popup_backRadio", num_pais, num);
	
	Config.isChangedRadio = true;
}


/*******************************************************************************
 * controller_deleteRadio - funcao que controla a remocao de uma radio
 *******************************************************************************/
function controller_deleteRadio() 
{
	var box = document.getElementById("popup_backRadio");
	if (box.length==0) return;
	
	// Remove da struct radios
	var numradio = box.options[box.selectedIndex].value;
	Radio.del(numradio);
	
	// Limpa e preenche novamente o popup das radios
	var pais_root = document.getElementById("popup_backCountry");
	var num_pais = pais_root[pais_root.selectedIndex].value;
	Radio.populate("popup_backRadio", num_pais, -1);

	// Atualiza caixa de texto txt_radioName
	if (box.length > 0)
		Controller.radioChangeBackPopup(); //null);
	else {
		document.getElementById("txt_radioName").value = "";
		document.getElementById("txt_radioURL").value = "";
	}
	
	// VRF se existe algum botao de memoria apontando para ela
	Memory.checkButtons(numradio);
	
	Config.isChangedRadio = true;
}


/*******************************************************************************
 * controller_changeBackRadio - Evento onChange na popup_backRadio.
 *                    Atualiza as caixas de texto com o nome e o link da radio
 *******************************************************************************/
function controller_changeBackRadio()
{
	var box = document.getElementById("popup_backRadio");
	var chosen = box.selectedIndex;
	
	var numradio = box.options[chosen].value;
	// Atualiza caixas de texto
	document.getElementById("txt_radioName").value = Radio.getAttrib(numradio, "nome");
	document.getElementById("txt_radioURL").value = Radio.getAttrib(numradio, "link");
}


/*******************************************************************************
 * controller_changeFrontRadio - Evento onChange na popup_plRadio.
 *                  Atualiza os botoes de memorias da radio selecionada e se
 *                  a opcao de tocar as radios automaticamente estiver 
 *                  selecionada, toca a radio
 *******************************************************************************/
function controller_changeFrontRadio() 
{
	Memory.refreshButtons();
	Memory.onForCurrentRadio();
	Controller.setPopupText();
	
	box_radio = document.getElementById("popup_plRadio");
	Memory.radioSelected = box_radio.options[box_radio.options.selectedIndex].value;
	
	// VRF se eh para iniciar a radio automaticamente
	if (Config.cfgAutoplay)
		Streaming.play();
}


/***************************** CONTROLLER-CONFIG *******************************/

/*******************************************************************************
 * controller_saveAll - salva as configuracoes: paises, radios, versao da 
 *					    configuracao, memoria e opcoes gerais
 *******************************************************************************/
function controller_save()
{
	if (!window.widget)
		return;
	
	if (Config.isChangedRadio) {
		Panel.setStatusMessage("Wait! Saving Radios...", false, "bold");
		Panel.showStatusMessage();
		
		setTimeout("Controller.subSave()", 400);
	}
	else Controller.subSave();
}

function controller_subSave() {
	// Salva configuracoes
	if (Config.isChangedConfig)
		Config.save();
	
	// Salva Paises
	if (Config.isChangedCountry)
		Country.save();
	
	// Salva radios
	if (Config.isChangedRadio) {
		Radio.save();
		if (Panel.statusTime)
			clearTimeout(Panel.statusTime);
		Panel.hideStatusMessage();
	}
	
	// Salva as memorias
	//Memory.save();
	
	Config.isChangedConfig=false;
	Config.isChangedCountry=false;
	Config.isChangedRadio=false;
}


/*******************************************************************************
 * controller_loadAll - carrega as listas de paises, radios e memorias e armazena
 *						os dados nas structs 'paises', 'radios' e 'memory'.
 *******************************************************************************/
function controller_loadAll()
{
	if(!window.widget)
		return;
	
	// Carrega paises
	Country.load();
	
	// Parse das radios
	Radio.load();
	
	// Carrega as memorias
	Memory.load();
	
	// Carrega as configuracoes
	Config.load();
}


/*******************************************************************************
 * controller_openHomePage - Chama o browser default para a abrir homepage do WRW
 *******************************************************************************/
function controller_openHomePage() 
{
	widget.openURL("http://worldradio.bazyl.com.br");
}


/***************************** CONTROLLER-POPUP ********************************/

/*******************************************************************************
 * controller_clearPopup - Limpa o popup especificado em obj_popup
 *******************************************************************************/
function controller_clearPopup(obj_popup) {
	// Limpa o popup
	var popup = document.getElementById(obj_popup);
	while(popup.length > 0) {
		popup.options[popup.length-1] = null;
	}
	popup.disabled = true;
}


/*******************************************************************************
 * controller_setPopupText - Ajusta a apresentacao dos popup atualizando o 
 *							 texto da imagem que estah sobre o popup.
 *******************************************************************************/
function controller_setPopupText()
{
	// Ajusta a apresentacao dos popups (_img)
	box_pais = document.getElementById("popup_plCountry");
	if (box_pais.length==0) return;
	document.getElementById("popup_plCountry_img").object.textElement.innerText = box_pais.options[box_pais.options.selectedIndex].text;
	
	box_radio = document.getElementById("popup_plRadio");
	if(box_radio.length>0) 
		document.getElementById("popup_plRadio_img").object.textElement.innerText = box_radio.options[box_radio.options.selectedIndex].text;
	else
		document.getElementById("popup_plRadio_img").object.textElement.innerText = "Empty";
}


/***************************** CONTROLLER-UPDATE *******************************/

/*******************************************************************************
 * controller_cancelUpdate - Cancela o download de atualizao das radios
 *******************************************************************************/
function controller_cancelUpdate()
{
	myXMLRequest=false;
	// Oculta o painel de atualizacao
	Panel.frontHide();
	// Habilita o botao Info
	document.getElementById("info").style.display="block";
}


/*******************************************************************************
 * controller_acceptUpdate - realiza a atualizao das radios
 *******************************************************************************/
function controller_acceptUpdate()
{
	// Oculta o painel de atualizacao
	Panel.frontHide();
	// Habilita o botao Info
	document.getElementById("info").style.display="block";

	Panel.setStatusMessage("Wait! Updating...", false, "bold");
	Panel.showStatusMessage();
	
	setTimeout("Controller.updateRadios()", 350);
}


/*******************************************************************************
 * controller_updateRadios - processa novamente o arquivo XML, mas agora 
 *						 	 realizando as atualizacoes
 *******************************************************************************/
function controller_updateRadios() {
	//Controller.downloadStep=2;
	Controller.processUpdateXMLRequest(myXMLRequest.responseXML);
}


/*******************************************************************************
 * controller_checkWidgetUpdate - verifica a existencia de atualizacao do widget 
 *******************************************************************************/ 
function controller_checkWidgetUpdate() 
{
	// Salva configuracoes e exibe a frente do widget
	//if (Config.isChanged) {
		Config.getParameters();
		Controller.save();
	//}
	showFront();
	
	// Verifica update do widget
	Controller.checkUpdates("widget");
}


/*******************************************************************************
 * controller_checkRadioUpdate - faz o download de todas as radios do site
 *******************************************************************************/ 
function controller_checkRadioUpdate()
{
	// Salva configuracoes e exibe a frente do widget
	//if (Config.isChanged) {
		Config.getParameters();
		Controller.save();
	//}
	showFront();
	
	// Verifica updates das radios
	Controller.checkUpdates("radios");
}


/*******************************************************************************
 * controller_checkUpdates - Conecta com o site do WorldRadio e faz o download
 *                 do arquivo 'wrw.xml' para verificar a versao mais
 *                 atual do widget
 *******************************************************************************/
function controller_checkUpdates(what)
{
	if (what=="widget") {
		//var URL = "http://192.168.1.2/~bazyl/wrw.xml";
		var URL = "http://www.bazyl.com.br/wrw/download/wrw.xml";
		//Controller.downloadStep=0;
		Controller.downloadFunction = VRF_WDGT;
	}
	else {
		//var URL = "http://192.168.1.2/~bazyl/radios.xml";
		var URL = "http://www.bazyl.com.br/wrw/download/radios.xml";
		//Controller.downloadStep=1;
		Controller.downloadFunction = VRF_RADIO;

		// Exibe msg de status (com scroll)
		Panel.setStatusMessage("Downloading control file", true, "bold");
		Panel.showStatusMessage();
	}
			
	myXMLRequest = false;
	myXMLRequest = new XMLHttpRequest();
	myXMLRequest.onload=Controller.receiveXMLRequest;
	myXMLRequest.overrideMimeType("text/xml");
	myXMLRequest.open("GET", URL);
	myXMLRequest.setRequestHeader("Cache-Control", "no-cache");
	//myXMLRequest.onreadystatechange = receiveXMLRequest;
	myXMLRequest.send(null);
}


/*******************************************************************************
 * controler_receiveXMLRequest - Processa a resposta recebida do site do WRW
 *******************************************************************************/
function controller_receiveXMLRequest()
{
	if (myXMLRequest.readyState == 4) {
		if (myXMLRequest.status == 200) {		// data request status is "OK"
			if (Controller.downloadFunction == VRF_WDGT)	// Verify widget update
				Controller.processXMLRequest(myXMLRequest.responseXML);
			else											// Verify new Radios
				Controller.processRadiosXMLRequest(myXMLRequest.responseXML);
		}
		else {
			if (myXMLRequest.status == 404) {
				// Exibe msg de erro (sem scroll)
				Panel.setStatusMessage("File not found in server!", false, "default");
				Panel.showStatusMessage();
				Panel.statusTime = setTimeout("Panel.hideStatusMessage()", 4000);
				return;
			}				
			alert("Error:"+myXMLRequest.status);
			// handleFailedXMLRequest();
		}
	}
}


/*******************************************************************************
 * controller_processXMLRequest - Processa o conteudo do arquivo XML recebido
 * Entrada: xml - arquivo XML
 *******************************************************************************/
function controller_processXMLRequest(xml) {
	var xml_radios, xml_widget;

	if (xml) {
		var xml_radios = Parser.findChild(xml, 'wrw');
		if(!xml_radios) return;
		for(var item = xml_radios.firstChild; item != null; item = item.nextSibling) {
			if (item.nodeName == 'widget') {
				var xml_widget = Parser.findChild(item, 'id');
				var newversion = xml_widget.firstChild.data;
			}
		}
	}
	Controller.updateWidget(newversion);
}


/*******************************************************************************
 * controller_processRadiosXMLRequest - Processa o conteudo do arquivo XML de radios
 * Entrada: xml - arquivo XML
 *******************************************************************************/
function controller_processRadiosXMLRequest(xml) {
	var cont_newcountry=0;	//
	var cont_newradio=0;
	var cont_updradio=0;
	var cont_delradio=0;
	//var cfgversao = Config.cfgVersionConfig;		// versao da config atual
	var num_radio; //
	
	if (xml) {
		var xml_radios = Parser.findChild(xml, 'radios');
		for(var item = xml_radios.firstChild; item != null; item = item.nextSibling) {
			// Processa pais
			if (item.nodeName == 'country') {
				var xml_country = Parser.findChild(item, 'name');
				var nome = xml_country.firstChild.data;

				var pais_atual = Country.search(nome);	// Procura pais
				if(pais_atual==-1)					// Pais nao encontrado
					cont_newcountry++;
			}	// Processa Radio
			else if (item.nodeName == 'radio') {
				var xml_radio_name = Parser.findChild(item, 'name');
				var nome = xml_radio_name.firstChild.data;
				
				var xml_radio_url = Parser.findChild(item, 'url');
				var new_url = xml_radio_url.firstChild.data;

				var xml_radio_ope = Parser.findChild(item, 'op');
				var ope = xml_radio_ope.firstChild.data;

				// Fazer busca apenas pelo nome para poder atualizar o link
				if ((num_radio=Radio.search(nome))==-1) {	// Radio nao existe
					if (ope=="ADD") cont_newradio++;
				}
				else {		// Radio jah existe, verifica operacao (atualizar link/Del/Ren)
					if (ope=="DEL") cont_delradio++;
					else {
						var url = Radio.getAttrib(num_radio, "link");
						if (url != new_url) {
							var pais = Radio.getAttrib(num_radio, "pais");
							if (pais==pais_atual) cont_updradio++;
						}
					}
				}
			}
		}
	}

	// Oculta a linha de media_status ("Downloading control file...")
	Panel.hideStatusMessage();
	// Existe atualizacao, entao pergunta se o usuario deseja atualizar
	if(cont_newradio>0 || cont_newcountry>0 || cont_updradio>0 || cont_delradio>0) {
		var str1 = "";
		var virgula = false;
		str1 = "There are new updates:<br>";
		
		if (cont_newradio > 0) {
			str1 = str1 + cont_newradio + " radio";
			if (cont_newradio > 1) str1=str1+"s";
			str1=str1+"	to ADD";
			virgula=true;
		}
		if (cont_updradio > 0) {
			if (virgula) str1=str1+";<br>";
			str1 = str1 + cont_updradio + " radio";
			if (cont_updradio > 1) str1=str1+"s";
			str1=str1+" to UPD";
			virgula=true;
		}
		if (cont_delradio > 0) {
			if (virgula) str1=str1+";<br>";
			str1 = str1 + cont_delradio + " radio";
			if (cont_delradio > 1) str1=str1+"s";
			str1=str1+" to DEL";
			virgula=true;
		}
		if (cont_newcountry > 0) {
			if (virgula) str1=str1+"; and<br>";
			str1 = str1 + cont_newcountry + " countr";
			if (cont_newcountry > 1) str1=str1+"ies to ADD";
			else str1=str1+"y to ADD";
		}
		str1=str1+".<br><br>Do you want proceed it?";
		document.getElementById("update_lbl").innerHTML = str1;
		
		// Exibe o painel com o frame de atualizacao
		Panel.frontShow("yesno");
		// Desabilita o botao Info
		document.getElementById("info").style.display="none";
	}
	else {
		Panel.setStatusMessage("No new updates available.", false, "default");
		Panel.showStatusMessage();
		Panel.statusTime = setTimeout("Panel.hideStatusMessage()", 4000);
	}
}


/*******************************************************************************
 * controller_processUpdateXMLRequest - Atualiza as radios/paises de acordo com
 *										o arquivo xml lido do servidor
 * Entrada: xml - arquivo XML
 *******************************************************************************/
function controller_processUpdateXMLRequest(xml) {
	var cont_newcountry=0;
	var cont_newradio=0;
	var cont_updradio=0;
	var cont_delradio=0;
	var num_radio;

	if (xml) {
		var xml_radios = Parser.findChild(xml, 'radios');
		for(var item = xml_radios.firstChild; item != null; item = item.nextSibling) {
			// Processa pais
			if (item.nodeName == 'country') {
				var xml_country = Parser.findChild(item, 'name');
				var nome = xml_country.firstChild.data;

				pais_atual = Country.search(nome);	// Procura pais
				if(pais_atual==-1) {				// Pais nao encontrado
					// Add new country
					pais_atual = Country.add(nome, -1);		// -1 indica que eh uma nova numeracao
					cont_newcountry++;
				}
			}	// Processa Radio
			else if (item.nodeName == 'radio') {
				var xml_radio_name = Parser.findChild(item, 'name');
				var nome = xml_radio_name.firstChild.data;

				var xml_radio_url = Parser.findChild(item, 'url');
				var url = xml_radio_url.firstChild.data;
				
				var xml_radio_ope = Parser.findChild(item, 'op');
				var ope = xml_radio_ope.firstChild.data;
				
				// Fazer busca apenas pelo nome para poder atualizar o link
				if ((num_radio=Radio.search(nome))==-1) {	// Radio nao existe
					// Executa operacao (Add/Del/Ren) radio
					// -1 indica que é uma nova numeracao
					Radio.update(nome, url, pais_atual, -1, ope);
					if (ope=="ADD") cont_newradio++;
				}
				else {		// Radio jah existe, executa operacao (atualiza o link/Del/Ren)
					if (Config.cfgAllowUpdate) {
						if (ope == "DEL") cont_delradio++;
						else {
							var current_url = Radio.getAttrib(num_radio, "link");
							if (url != current_url) {
								pais = Radio.getAttrib(num_radio, "pais");
								if (pais == pais_atual) {
									Radio.update(nome, url, pais_atual, num_radio, ope);
									cont_updradio++;
									//alert("NUM="+num_radio+" NOME="+nome+" OPE="+ope);
								}
							}
						}
					}
				}
//alert("Parse: ("+num_radio+") "+nome);
			}	
			// Processa versao da configuracao de radios
			/* Retirado. Por enquanto, está sem sentido usá-la...
			else if (item.nodeName == 'version') {
				var xml_version = Parser.findChild(item, 'id');
				var versao = xml_version.firstChild.data;
				
					if(versao > Config.cfgVersionConfig)
						Config.cfgVersionConfig = versao;
			}
			*/
		}
	}

	// Oculta a linha de media_status ("Downloading control file...")
	//document.getElementById("media_status").style.display = "none";
	document.getElementById("media_status").innerHTML = "";
	// Informa atualização
	if(cont_newradio>0 || cont_newcountry>0 || cont_updradio>0 || cont_delradio>0) {
		var str1 = "";
		var virgula = false;
		str1 = "Configuration updated:<br>";
		
		if (cont_newradio > 0) {
			str1 = str1 + cont_newradio + " radio";
			if (cont_newradio > 1) str1=str1+"s";
			str1=str1+"	Added";
			virgula=true;
		}
		if (cont_updradio > 0) {
			if (virgula) str1=str1+";<br>";
			str1 = str1 + cont_updradio + " radio";
			if (cont_updradio > 1) str1=str1+"s";
			str1=str1+" Updated";
			virgula=true;
		}
		if (cont_delradio > 0) {
			if (virgula) str1=str1+";<br>";
			str1 = str1 + cont_delradio + " radio";
			if (cont_delradio > 1) str1=str1+"s";
			str1=str1+" Deleted";
			virgula=true;
		}
		if (cont_newcountry > 0) {
			if (virgula) str1=str1+"; and<br>";
			str1 = str1 + cont_newcountry + " countr";
			if (cont_newcountry > 1) str1=str1+"ies added";
			else str1=str1+"y added";
		}
		str1=str1+".";
	}
	else {	// Não foi realizada nenhuma atualizacao
		var str1="No update was realized.<br><br>";
		str1=str1+"Verify the configuration item 'Allow update link / delete radio'.";
	}
	
	document.getElementById("update_lbl").innerHTML = str1;
	
	// Oculta o painel de informação (Neste ponto ele estaria apresentando: "Wait! Updating...")
	//Panel.frontHide();
	
	// Oculta os botoes yes/no
	//document.getElementById("btn_nolbl").style.display = "none";
	//document.getElementById("btn_no").style.display = "none";
	//document.getElementById("btn_yeslbl").style.display = "none";
	//document.getElementById("btn_yes").style.display = "none";
	
	// Exibe o painel com o frame de atualizacao
	Panel.frontShow("ok");
	
	// Desabilita o botao Info
	//document.getElementById("info").style.display="none";
	
	// Oculta o painel apos 6 segundos
	//setTimeout("Panel.frontHide()", 6000);
//	alert("Radios gravadas="+radios[0].num);
	// Salva nova configuracao
	Controller.save();
		
	// Configura a parte frontal (popup de radios/paises)
	config_front(-1);
}


/*******************************************************************************
 * controller_updateWidget - Verifica se a versao do widget do usuario eh mais antiga
 *                 que a existente no servidor (constante no arquivo wrw.xml) e
 *                 exibe a mensagem correspondente.
 * Entrada: newversion - numero da versao lida do arquivo wrw.xml
 *******************************************************************************/
function controller_updateWidget(newversion)
{
	// Mostra a informacao da atualizacao
	if (newversion > Config.cfgVersionWidget) {
		var str1 = "New update (v"+newversion+") available!";
		Panel.setStatusMessage(str1, false, "default");
	}
	else Panel.setStatusMessage("No Widget update found.", false, "default");

	Panel.showStatusMessage();
	Panel.statusTime = setTimeout("Panel.hideStatusMessage()", 4000);
}


/*********************************** PARSER ************************************/

/*******************************************************************************
 * parser_checkString - verifica se a string esta preenchida (true)
 *						ou em branco (false)
 *******************************************************************************/
function parser_checkString(str1)
{
	for (var i=0; i<=str1.length-1; i++)
		if (str1.charCodeAt(i) >= 33) return(true);
	return(false);
}


/*******************************************************************************
 * parser_findChild - Scans the children of a given DOM element for a node 
 *                    matching nodeName.
 *******************************************************************************/
function parser_findChild (element, nodeName)
{
	var child;
	
	for (child = element.firstChild; child != null; child = child.nextSibling)
		if (child.nodeName == nodeName)
			return child;
	return null;
}


/*******************************************************************************
 * parser_decodeURL - Faz o parse da URL arrastada para o widget e extrai as
 *		              informacoes de nome, link e pais da radio
 *******************************************************************************/
function parser_decodeURL(uri) {
	// Verifica se a URL comeca com 'wrw:'
	if(uri.substring(0,4)!="wrw:") return(-1);

	var cont_add=0;
	var cont_upd=0;
	var cont_pais=0;
	
	while (uri.length > 0) {
		// Racupera o nome do pais
		var pos=uri.indexOf(":");
		if(pos==-1) return(-1);
		uri=uri.substring(pos+1);		// Corta o wrw:
		var nome_pais = uri.substring(0, uri.indexOf("::"));
		nome_pais = Parser.processURL(nome_pais);
		// Recupera o nome da radio
		pos=uri.indexOf("::");
		if(pos==-1) return(-1);
		uri=uri.substring(pos+2);
		var nome_radio = uri.substring(0, uri.indexOf("::"));
		nome_radio = Parser.processURL(nome_radio);
	    // Recupera o link da radio
		uri=uri.substring(uri.indexOf("::")+2);
		pos=uri.indexOf(";;");
		if(pos==-1 && uri!="") {
			var link_radio=uri;
			uri="";
		}
		else {
			var link_radio=uri.substring(0, uri.indexOf(";;"));
			uri = uri.substring(pos+2);
		}
		
		// Se pais nao existe, entao adiciona
		var num_pais;
		if ((num_pais=Country.search(nome_pais))==-1) {
			num_pais = Country.add(nome_pais, -1);
			cont_pais++;
			Config.isChangedCountry = true;
		}
	
		// Se radio nao existe, entao adiciona
		var num_radio;
		if ((num_radio=Radio.search(nome_radio))==-1) {
			Radio.add(nome_radio, link_radio, num_pais, -1);
			cont_add++;
			Config.isChangedRadio = true;
		}
		else {
			// Radio jah existe, entao atualiza a URL
			if (Config.cfgAllowUpdate) {
				var current_url = Radio.getAttrib(num_radio, "link");
				if (link_radio != current_url) {
					num_pais = Radio.getAttrib(num_radio, "pais");
					Radio.del(num_radio);
					Radio.add(nome_radio, link_radio, num_pais, num_radio);
					cont_upd++;
					Config.isChangedRadio = true;
				}
			}
		}
	}
	// Usado para atualizar os popups
	config_front(-1);

	// Informa atualização
	if(cont_add>0 || cont_upd>0 || cont_pais>0) {
		var str1 = "";
		var virgula = false;
		str1 = "Configuration updated:<br>";
		
		if (cont_add > 0) {
			str1 = str1 + cont_add + " radio";
			if (cont_add > 1) str1=str1+"s";
			str1=str1+"	Added";
			virgula=true;
		}
		if (cont_upd > 0) {
			if (virgula) str1=str1+";<br>";
			str1 = str1 + cont_upd + " radio";
			if (cont_upd > 1) str1=str1+"s";
			str1=str1+" Updated";
			virgula=true;
		}
		if (cont_pais > 0) {
			if (virgula) str1=str1+"; and<br>";
			str1 = str1 + cont_pais + " countr";
			if (cont_pais > 1) str1=str1+"ies added";
			else str1=str1+"y added";
		}
		str1=str1+".";
	}
	else {	// Não foi realizada nenhuma atualizacao
		if (Config.cfgAllowUpdate) {
			Panel.setStatusMessage("Radios already updated!", false, "default");
			Panel.showStatusMessage();
			Panel.statusTime = setTimeout("Panel.hideStatusMessage()", 4000);
			return(0);
		}
		else {	
			var str1="No update was realized.<br><br>";
			str1=str1+"Verify the configuration item 'Allow update link / delete radio'.";
		}
	}
	
	document.getElementById("update_lbl").innerHTML = str1;

	// Exibe a mensagem definida acima informando quantas radios
	// foram adicionadas ou atualizadas
	Panel.frontShow("ok");
	//document.getElementById("update_lbl").innerHTML = str;
	//setTimeout("Panel.frontHide()", 10000);
	
	return(0);
}


/*******************************************************************************
 * parseURL - Usado para substituir o %20 inserido pelo Safari por " " (espaço)
 *******************************************************************************/
function parser_processURL(str1) {
	var lupe=true;
	
	while(lupe) {
		var pos = str1.indexOf("%20");
		if (pos!=-1) {
			var part_ini = str1.substring(0, str1.indexOf("%20"));
			var part_fim = str1.substring(str1.indexOf("%20")+3);
			str1 = part_ini + " " + part_fim;
		}
		else lupe=false;
	}
	return(str1);
}

function parser_correctString(str1) {
	if (!str1) return("");
	if (str1.length<=0) return ("");
	
	while(str1.charAt(0)==" ") {
		if (str1.length==1) {
			str1="";
			break;
		}
		else str1 = str1.substring(1, str1.length-1);
	}

	while(str1.charAt(str1.length-1)==" ") {
		if (str1.length==1) {
			str1="";
			break;
		}
		else str1=str1.substring(0, str1.length-1);
	}

	return(str1);
}


function parser_validateMetadata(vrfstr, contador) {
	var sep = "";
	str1 = Parser.correctString(vrfstr);
	
	if (str1.length>2 && contador<3) {
		if (contador>0) sep=" :: ";
		
		if (contador==1) {
			var campo1=Display.stringScrollMetadata;
			var campo2="";
		}
		else {
			var campo1 = Display.stringScrollMetadata.substring(0, Display.stringScrollMetadata.indexOf(" :: "));
			var campo2 = Display.stringScrollMetadata.substring(Display.stringScrollMetadata.indexOf(" :: ")+4, Display.stringScrollMetadata.length);
		}
//		alert("1="+campo1+" 2="+campo2+ "Tentando="+str1+"(entrou="+vrfstr+")");
		
		//if (campo1.indexOf(str1)<0 && campo2.indexOf(str1)<0) {
		if (str1!=campo1 && str1!=campo2) {
			Display.stringScrollMetadata = Display.stringScrollMetadata + sep + str1;
			contador++;
		}
	}
	
	return(contador);
}
