/*******************************************************************************
 * WRW - WorldRadio Widget
 * by Alexandre Bazyl <wrw@bazyl.com.br>
 *
 * -- version 1.3   :: JUN/2008
 * -- version 1.1.4 :: OCT/2007
 * -- version 1.1.3 :: OCT/2007
 * -- version 1.1.2 :: SEP/2007
 * -- version 1.0b  :: AUG/2007
 *
 * wrmemo.js - funcoes relacionadas ao objeto Memory
 *			   Obs: o objeto Memory sera utilizado apenas para manter os
 *				    metodos. A relacao de memorias sera mantida em um vetor
 *
 *******************************************************************************/


/************************************ MEMORY ***********************************/
// Estados possiveis do botao M
var MEMDIS = 0;
var MEMSET = 1;
var MEMDEL = 2;
// Vetor de memorias
var memory = new Array;
// Objecto Memory e seus metodos
Memory = new Object();
Memory.load = memory_load;
Memory.save = memory_save;
Memory.enable = memory_enable;
Memory.showInfo = memory_showInfo;
Memory.setInfoColor = memory_setInfoColor;
Memory.delInfoColor = memory_delInfoColor;
Memory.processButtonClick = memory_processButtonClick;
Memory.set = memory_set;
Memory.del = memory_del;
Memory.get = memory_get;
Memory.refreshButtons = memory_refreshButtons;
Memory.onForCurrentRadio = memory_onForCurrentRadio;
Memory.checkButtons = memory_checkButtons;
// Atributos do objeto Memory
Memory.time = 0;
Memory.state = 0;
Memory.radioSelected = 0;


/*******************************************************************************
 * memory_enable - Chamado pela botao "M". Muda o estado das funcoes de memoria
 *				   e atualiza a apresentacao do botao M.
 *                 Os estados sao MEMDIS (get), MEMSET (set) e MEMDEL (del)
 *******************************************************************************/
function memory_enable() 
{
	var btnmemo = document.getElementById("btn_memory");

	// Se exitir alguma indicacao de memoria piscando entao pára
	if(Memory.time)
		clearTimeout(Memory.time);
	
	if (Memory.state == MEMDIS) {
		Memory.state = MEMSET;		// Passa para o estado de adicionar memoria
		btnmemo.src = "Images/btn_hmemory.png";
		Memory.showInfo("set", true);
		
	}
	else if (Memory.state == MEMSET) {
		Memory.state = MEMDEL;		// Passa para o estado de deletar memoria
		btnmemo.src = "Images/btn_delmem.png";
		Memory.setInfoColor(false);
		Memory.showInfo("del", true);
	}
	else {
		Memory.state = MEMDIS;		// Passa para o estado inativo (get)
		btnmemo.src = "Images/btn_memory.png";
		Memory.delInfoColor(false);
	}
}


/*******************************************************************************
 * memory_processButtonClick - Pressionamento de um dos botoes "memX" (X = 1..10).
 *             VRF se eh para 'set' ou 'get' (definido por Memory.state)
 *             a radio referente ao botao de memoria pressionado
 * Entrada: numbtn: numero do botao
 *******************************************************************************/
function memory_processButtonClick(numbtn)
{
	if (Memory.state == MEMSET)			// set memory
		Memory.set(numbtn);
	else if (Memory.state == MEMDEL)	// del memory
		Memory.del(numbtn);
	else Memory.get(numbtn);			// get memory
}


/*******************************************************************************
 * memory_set - atribui a radio atual a um botao de memoria
 * Entrada: nummemo: numero do botao
 *******************************************************************************/
function memory_set(nummemo)
{
	var exists = false;
	var box = document.getElementById("popup_plRadio");
	if (box.length == 0) return;
	
	var numradio = box.options[box.selectedIndex].value;
	for (var i=1; i<=10; i++)
		if (memory[i]==numradio)
			exists = true;
	
	if (exists) {
		Panel.setStatusMessage("This radio already in memory!", false, "default");
		Panel.showStatusMessage();
		Panel.statusTime = setTimeout("Panel.hideStatusMessage()", 4000);
	}
	else {
		memory[nummemo]=numradio;
		// Salva as memorias
		Memory.save();
	}
	
	Memory.state = MEMDIS;		// Sai do estado de set e volta ao estado de get
	document.getElementById("btn_memory").src = "Images/btn_memory.png";
	Memory.refreshButtons();	// atualiza a apresentacao de todos os botoes
	Memory.onForCurrentRadio();	// hilight o botao referente a radio atualmente selecionada
	
	// Pára de piscar a indicacao de MEMSET
	if(Memory.time)
		clearTimeout(Memory.time);
	Memory.setInfoColor(false);
}


/*******************************************************************************
 * memory_get - recupera a radio ligada a um botao de memoria
 * Entrada: nummemo: numero do botao
 *******************************************************************************/
function memory_get(nummemo)
{
	if(memory[nummemo]==0) return;
	Memory.radioSelected = memory[nummemo];
	
	// Preenche os popups
	var chkpais = Radio.getAttrib(memory[nummemo], "pais");
	var numpais = Country.populate("popup_plCountry", chkpais);		// Populate a popup de paises
	Radio.populate("popup_plRadio", numpais, memory[nummemo]);		// Populate a popup de radios
	Memory.refreshButtons();					    // Atualiza a aparencia dos botoes de memoria
	Memory.onForCurrentRadio();		
	Controller.setPopupText();
	
	var btnname = "btn_mem" + nummemo;
	// Hilight o botao de memoria selecionado
	document.getElementById(btnname).src = "Images/btn_hmem" + nummemo + ".png";
	
	// VRF se eh para iniciar a radio automaticamente
	if (Config.cfgAutoplay)
		setTimeout("playradio(null)", 250);
}


/*******************************************************************************
 * memory_del - apaga a memoria de um determinado botao
 * Entrada: nummemo: numero do botao a ser apagada a memoria
 *******************************************************************************/
function memory_del(nummemo)
{
	memory[nummemo]=0;
	Memory.state = MEMDIS;
	Memory.refreshButtons();
	Memory.onForCurrentRadio();
	document.getElementById("btn_memory").src = "Images/btn_memory.png";

	// Salva as memorias
	Memory.save();
	
	// Pára de piscar a indicacao de MEMDEL
	if(Memory.time)
		clearTimeout(Memory.time);
	Memory.delInfoColor(false);
}


/*******************************************************************************
 * memory_refreshButtons - atualiza a aparencia de todos os botoes de memoria,
 *						   conforme possui ou nao uma radio atribuida a ele.
 *******************************************************************************/
function memory_refreshButtons()
{
	var btnname;
	for (var i=1; i<=10; i++) {
		btnname = "btn_mem" + i;
		var btnmemo = document.getElementById(btnname);
		if (memory[i]==0)	// Mostra botao de memoria desabilitado
			btnmemo.src = "Images/btn_dismem" + i + ".png";
		else				// Mostra botao de memoria habilitado
			btnmemo.src = "Images/btn_mem" + i + ".png";
	}
}


/*******************************************************************************
 * memory_onForCurrentRadio - VRF se existe algum botao de memoria atribuido a 
 *							  radio selecionada. Se tiver, este botao ficara 
 *							  destacado
 *******************************************************************************/
function memory_onForCurrentRadio() {
	var box = document.getElementById("popup_plRadio");
	if(box.length == 0) return;
	var selradio = box.options[box.selectedIndex].value;	// Numero da radio selecionada
	
	for (var i=1; i<=10; i++)
		if (memory[i] == selradio) {
			var btnname = "btn_mem" + i;
			// Mostra botao de memoria selecionado
			document.getElementById(btnname).src = "Images/btn_hmem" + i + ".png";
		}
}


/*******************************************************************************
 * memory_showInfo - Utilizada para piscar a indicacao MEMSET ou MEMDEL no display.
 *                   Utiliza a variavel Memory.time para controlar o timer da piscada.
 * Entrada: memopt - indica se eh para exibir MEMSET (set) ou MEMDEL (del)
              step - indica o estado do label: exibir (true) ou apagar (false)
 *******************************************************************************/
function memory_showInfo(memopt, step)
{
	if (Memory.time)
		clearTimeout(Memory.time);

	var next = !step;		
	if(memopt=="set")
		Memory.setInfoColor(step);
	else
		Memory.delInfoColor(step);

	var str1 = "Memory.showInfo(\""+memopt+"\","+next+")";
	Memory.time = setTimeout(str1, 500);	// inverte a "piscada" após 0,5 segundos
}


/*******************************************************************************
 * memory_setInfoColor - controla a apresentacao da indicacao MEMSET
 * Entrada: opc - indica se eh para exibir MEMSET (true) ou apagar (false)
 *******************************************************************************/
function memory_setInfoColor(opc) 
{
	if (opc)	// Display
		document.getElementById("lbl_mem_set").style.color = "#00BF00";
	else		// Hide
		document.getElementById("lbl_mem_set").style.color = "#001F00";
}


/*******************************************************************************
 * memory_delInfoColor - controla a apresentacao da indicacao MEMDEL
 * Entrada: opc - indica se eh para exibir MEMDEL (true) ou apagar (false)
 *******************************************************************************/
function memory_delInfoColor(opc) 
{
	if (opc)	// Display
		document.getElementById("lbl_mem_del").style.color = "#BF0000";
	else		// Hide
		document.getElementById("lbl_mem_del").style.color = "#1F0000";
}


/*******************************************************************************
 * memory_load - carrega as memorias para o vetor memory
 *******************************************************************************/
function memory_load() {
	// Zera as memorias
	for (var i=0; i<=10; i++) 
		memory[i]=0;
	
	if(window.widget)
		var strmem = widget.preferenceForKey("memory");
	else return;
	
	i=1;
	if (strmem > "") {
		var num;	
		while (strmem.length > 1) {
			num = strmem.substring(0, strmem.indexOf(';'));
			strmem = strmem.substring(strmem.indexOf(';')+1);
			memory[i] = Math.abs(num);
			i++;
		}
	}
	for (var j=i; j<=10; j++) 
		memory[j]=0;
}


/*******************************************************************************
 * memory_save - salva as configuracoes dos botoes de memoria
 *******************************************************************************/
function memory_save()
{
	// Salva as memorias
	var strmem = "";
	for (var i=1; i<=10; i++)
		strmem = strmem + memory[i] + ";"
		
	if(window.widget)
		widget.setPreferenceForKey(strmem, "memory");
}


/*******************************************************************************
 * memory_checkButtons - VRF se exite algum botao de memoria apontando para a 
 *						 radio recem deletada e apaga esta memoria
 * Entrada: numradio: numero da radio deletada
 *******************************************************************************/
function memory_checkButtons(numradio) {
	for (var i=1; i<=10; i++)
		if (memory[i] == numradio)
			memory[i]=0;
}
